@@ -1,5 +1,5 @@
 /*
-* "$Id: pap.c 6090 2006-11-14 16:35:27Z mike $"
+* "$Id: pap.c 7013 2007-10-10 21:25:29Z mike $"
 *
 *  Copyright 2004 Apple Computer, Inc. All rights reserved.
 * 
@@ -49,24 +49,25 @@
 *
 * Contents:
 *
-*  main()		- Send a file to the specified Appletalk printer.
-*  listDevices()	- List all LaserWriter printers in the local zone.
-*  printFile()		- Print file.
-*  papOpen()		- Open a pap session to a printer.
-*  papClose()		- Close a pap session.
-*  papWrite()		- Write bytes to a printer.
-*  papCloseResp()	- Send a pap close response.
-*  papSendRequest()	- Fomrat and send a pap packet.
-*  papCancelRequest()	- Cancel a pending pap request.
-*  statusUpdate()	- Print printer status to stderr.
-*  parseUri()		- Extract the print name and zone from a uri.
-*  addPercentEscapes()	- Encode a string with percent escapes.
-*  removePercentEscapes	- Remove percent escape sequences from a string.
-*  nbptuple_compare()	- Compare routine for qsort.
-*  okayToUseAppleTalk() - Returns true if AppleTalk is available and enabled.
-*  packet_name()	- Returns packet name string.
-*  connectTimeout()	- Returns the connect timeout preference value.
-*  signalHandler()	- handle SIGINT to close the session before quiting.
+*  main()		 - Send a file to the specified Appletalk printer.
+*  listDevices()	 - List all LaserWriter printers in the local zone.
+*  printFile()		 - Print file.
+*  papOpen()		 - Open a pap session to a printer.
+*  papClose()		 - Close a pap session.
+*  papWrite()		 - Write bytes to a printer.
+*  papCloseResp()	 - Send a pap close response.
+*  papSendRequest()	 - Fomrat and send a pap packet.
+*  papCancelRequest()	 - Cancel a pending pap request.
+*  sidechannel_request() - Handle side-channel requests.
+*  statusUpdate()	 - Print printer status to stderr.
+*  parseUri()		 - Extract the print name and zone from a uri.
+*  addPercentEscapes()	 - Encode a string with percent escapes.
+*  removePercentEscapes	 - Remove percent escape sequences from a string.
+*  nbptuple_compare()	 - Compare routine for qsort.
+*  okayToUseAppleTalk()  - Returns true if AppleTalk is available and enabled.
+*  packet_name()	 - Returns packet name string.
+*  connectTimeout()	 - Returns the connect timeout preference value.
+*  signalHandler()	 - handle SIGINT to close the session before quiting.
 */
 
 #include <config.h>
@@ -92,6 +93,8 @@
 
 #include <cups/cups.h>
 #include <cups/backend.h>
+#include <cups/sidechannel.h>
+#include <cups/i18n.h>
 
 #include <libkern/OSByteOrder.h>
 
@@ -167,6 +170,7 @@ static int papCloseResp(int sockfd, at_inet_t* dest, int xo, u_short tid,
 static int papSendRequest(int sockfd, at_inet_t* dest, u_char connID, 
 			  int function, u_char bitmap, int xo, int seqno);
 static int papCancelRequest(int sockfd, u_short tid);
+static void sidechannel_request();
 static void statusUpdate(char* status, u_char statusLen);
 static int parseUri(const char* argv0, char* name, char* type, char* zone);
 static int addPercentEscapes(const char* src, char* dst, int dstMax);
@@ -203,12 +207,7 @@ int main (int argc, const char * argv[])
 
   if (argc == 1 || (argc == 2 && strcmp(argv[1], "-discover") == 0))
   {
-    /* If listDevices() didn't find any devices or returns an error output a 
-    *  legacy style announcement.
-    *  
-    */
-    if (listDevices() <= 0)
-      puts("network pap \"Unknown\" \"AppleTalk Printer Access Protocol (pap)\"");
+    listDevices();
 
     return 0;
   }
@@ -238,7 +237,9 @@ int main (int argc, const char * argv[])
     /* Try to open the print file... */
     if ((fp = fopen(argv[6], "rb")) == NULL)
     {
-      fprintf(stderr, "ERROR: unable to open print file \"%s\": %s\n", argv[6], strerror(errno));
+      _cupsLangPrintf(stderr,
+                      _("ERROR: Unable to open print file \"%s\": %s\n"),
+		      argv[6], strerror(errno));
       return (1);
     }
 
@@ -285,7 +286,7 @@ static int listDevices(void)
   /* Make sure it's okay to use appletalk */
   if (!okayToUseAppleTalk())
   {
-    fprintf(stderr, "ERROR: AppleTalk disabled in System Preferences\n");
+    fprintf(stderr, "INFO: AppleTalk disabled in System Preferences\n");
     return -1;  /* Network is down */
   }
 
@@ -392,6 +393,7 @@ static int printFile(char* name, char* type, char* zone, int fdin, int fdout, in
   char	sockBuffer[4096 + 1];    /* Socket buffer with room for nul */
   char	atpReqBuf[AT_PAP_DATA_SIZE];
   fd_set readSet;
+  int	use_sidechannel;	/* Use side channel? */
 
   at_nbptuple_t	tuple;
   at_inet_t	sendDataAddr;
@@ -418,6 +420,22 @@ static int printFile(char* name, char* type, char* zone, int fdin, int fdout, in
   struct sigaction action;  /* Actions for POSIX signals */
 #endif /* HAVE_SIGACTION && !HAVE_SIGSET */
 
+ /*
+  * Test the side channel descriptor before calling papOpen() since it may open
+  * an unused fd 4 (a.k.a. CUPS_SC_FD)...
+  */
+
+  FD_ZERO(&readSet);
+  FD_SET(CUPS_SC_FD, &readSet);
+
+  timeout.tv_sec  = 0;
+  timeout.tv_usec = 0;
+
+  if ((select(CUPS_SC_FD+1, &readSet, NULL, NULL, &timeout)) >= 0)
+    use_sidechannel = 1;
+  else
+    use_sidechannel = 0;
+
   /* try to find our printer */
   if ((err = nbp_make_entity(&entity, name, type, zone)) != noErr)
   {
@@ -559,8 +577,12 @@ static int printFile(char* name, char* type, char* zone, int fdin, int fdout, in
   fileBufferNbytes = 0;
   fileTbytes = 0;
   fileEOFRead = fileEOFSent = false;
+
   maxfdp1 = MAX(fdin, gSockfd) + 1;
 
+  if (use_sidechannel && CUPS_SC_FD >= maxfdp1)
+    maxfdp1 = CUPS_SC_FD + 1;
+
   if (gStatusInterval != 0)
   {
     timeout.tv_usec  = 0;
@@ -580,6 +602,9 @@ static int printFile(char* name, char* type, char* zone, int fdin, int fdout, in
     if (fileBufferNbytes == 0 && fileEOFRead == false)
       FD_SET(fdin, &readSet);
 
+    if (use_sidechannel)
+      FD_SET(CUPS_SC_FD, &readSet);
+
     /* Set the select timeout value based on the next status interval */
     if (gStatusInterval != 0)
     {
@@ -604,6 +629,13 @@ static int printFile(char* name, char* type, char* zone, int fdin, int fdout, in
         nextStatusTime = time(NULL) + gStatusInterval;
     }
 
+   /*
+    * Check if we have a side-channel request ready...
+    */
+
+    if (use_sidechannel && FD_ISSET(CUPS_SC_FD, &readSet))
+      sidechannel_request();
+
     /* Was there an event on the input stream? */
     if (FD_ISSET(fdin, &readSet))
     {
@@ -1229,6 +1261,50 @@ int papCancelRequest(int sockfd, u_short tid)
 }
 
 
+/*
+ * 'sidechannel_request()' - Handle side-channel requests.
+ */
+
+static void
+sidechannel_request()
+{
+  cups_sc_command_t	command;	/* Request command */
+  cups_sc_status_t	status;		/* Request/response status */
+  char			data[2048];	/* Request/response data */
+  int			datalen;	/* Request/response data size */
+
+  datalen = sizeof(data);
+
+  if (cupsSideChannelRead(&command, &status, data, &datalen, 1.0))
+  {
+    fputs(_("WARNING: Failed to read side-channel request!\n"), stderr);
+    return;
+  }
+
+  switch (command)
+  {
+    case CUPS_SC_CMD_GET_BIDI:		/* Is the connection bidirectional? */
+	data[0] = 1;
+	cupsSideChannelWrite(command, CUPS_SC_STATUS_OK, data, 1, 1.0);
+	break;
+
+    case CUPS_SC_CMD_GET_STATE:		/* Return device state */
+	data[0] = CUPS_SC_STATE_ONLINE;
+	cupsSideChannelWrite(command, CUPS_SC_STATUS_OK, data, 1, 1.0);
+	break;
+
+    case CUPS_SC_CMD_DRAIN_OUTPUT:	/* Drain all pending output */
+    case CUPS_SC_CMD_SOFT_RESET:	/* Do a soft reset */
+    case CUPS_SC_CMD_GET_DEVICE_ID:	/* Return IEEE-1284 device ID */
+    default:
+	cupsSideChannelWrite(command, CUPS_SC_STATUS_NOT_IMPLEMENTED, 
+			     NULL, 0, 1.0);
+	break;
+  }
+  return;
+}
+
+
 #pragma mark -
 /*!
  * @function  statusUpdate
@@ -1284,9 +1360,9 @@ static int parseUri(const char* argv0, char* name, char* type, char* zone)
         *resourcePtr,
         *typePtr,
         *options,		/* Pointer to options */
-        optionName[255],	/* Name of option */
-        value[255],		/* Value of option */
-        *ptr;			/* Pointer into name or value */
+        *optionName,		/* Name of option */
+        *value,			/* Value of option */
+        sep;			/* Separator character */
   int   port;			/* Port number (not used) */
   int   statusInterval;		/* */
 
@@ -1314,53 +1390,54 @@ static int parseUri(const char* argv0, char* name, char* type, char* zone)
 
     while (*options != '\0')
     {
-      /*
+     /*
       * Get the name...
       */
-      for (ptr = optionName; *options && *options != '=' && *options != '+'; )
-        *ptr++ = *options++;
 
-      *ptr = '\0';
-      value[0] = '\0';
+      optionName = options;
 
-      if (*options == '=')
+      while (*options && *options != '=' && *options != '+' && *options != '&')
+        options ++;
+
+      if ((sep = *options) != '\0')
+        *options++ = '\0';
+
+      if (sep == '=')
       {
-        /*
+       /*
         * Get the value...
-        */
-        
-        options ++;
-        
-        for (ptr = value; *options && *options != '+';)
-          *ptr++ = *options++;
+	*/
 
-        *ptr = '\0';
-        
-        if (*options == '+')
-          options ++;
-      }
-      else if (*options == '+')
-      {
-        options ++;
-      }
+        value = options;
 
-      /*
+	while (*options && *options != '+' && *options != '&')
+	  options ++;
+
+        if (*options)
+	  *options++ = '\0';
+      }
+      else
+        value = (char *)"";
+
+     /*
       * Process the option...
       */
-      if (strcasecmp(optionName, "waiteof") == 0)
+
+      if (!strcasecmp(optionName, "waiteof"))
       {
-        /*
-        * Set the banner...
+       /*
+        * Wait for the end of the print file?
         */
-        if (strcasecmp(value, "on") == 0 ||
-          strcasecmp(value, "yes") == 0 ||
-          strcasecmp(value, "true") == 0)
+
+        if (!strcasecmp(value, "on") ||
+            !strcasecmp(value, "yes") ||
+            !strcasecmp(value, "true"))
         {
           gWaitEOF = true;
         }
-        else if (strcasecmp(value, "off") == 0 ||
-            strcasecmp(value, "no") == 0 ||
-            strcasecmp(value, "false") == 0)
+        else if (!strcasecmp(value, "off") ||
+                 !strcasecmp(value, "no") ||
+                 !strcasecmp(value, "false"))
         {
           gWaitEOF = false;
         }
@@ -1369,13 +1446,17 @@ static int parseUri(const char* argv0, char* name, char* type, char* zone)
           fprintf(stderr, "WARNING: Boolean expected for waiteof option \"%s\"\n", value);
         }
       }
-      else if (strcasecmp(optionName, "status") == 0)
+      else if (!strcasecmp(optionName, "status"))
       {
+       /*
+        * Set status reporting interval...
+	*/
+
         statusInterval = atoi(value);
-        if (value[0] < '0' || value[0] > '9' || 
-          statusInterval < 0)
+        if (value[0] < '0' || value[0] > '9' || statusInterval < 0)
         {
-          fprintf(stderr, "WARNING: number expected for status option \"%s\"\n", value);
+          fprintf(stderr, "WARNING: number expected for status option \"%s\"\n",
+	          value);
         }
         else
         {
@@ -1390,20 +1471,24 @@ static int parseUri(const char* argv0, char* name, char* type, char* zone)
   if (*resourcePtr == '/')
     resourcePtr++;
 
-        /* If the resource has a slash we assume the slash seperates the AppleTalk object
-         * name from the AppleTalk type. If the slash is not present we assume the AppleTalk
-         * type is LaserWriter.
-         */
-        typePtr = strchr(resourcePtr, '/');
-        if (typePtr != NULL) {
-            *typePtr++ = '\0';
-        } else {
-            typePtr = "LaserWriter";
-        }
+ /* If the resource has a slash we assume the slash seperates the AppleTalk object
+  * name from the AppleTalk type. If the slash is not present we assume the AppleTalk
+  * type is LaserWriter.
+  */
 
-  removePercentEscapes(hostname,    zone, NBP_NVE_STR_SIZE + 1);
-  removePercentEscapes(resourcePtr,  name, NBP_NVE_STR_SIZE + 1);
-  removePercentEscapes(typePtr,     type, NBP_NVE_STR_SIZE + 1);
+  typePtr = strchr(resourcePtr, '/');
+  if (typePtr != NULL)
+  {
+    *typePtr++ = '\0';
+  }
+  else
+  {
+    typePtr = "LaserWriter";
+  }
+
+  removePercentEscapes(hostname, zone, NBP_NVE_STR_SIZE + 1);
+  removePercentEscapes(resourcePtr, name, NBP_NVE_STR_SIZE + 1);
+  removePercentEscapes(typePtr, type, NBP_NVE_STR_SIZE + 1);
 
   return 0;
 }
