@@ -1,29 +1,17 @@
 /*
- * "$Id: adminutil.c 6625 2007-07-06 22:39:54Z mike $"
+ * "$Id: adminutil.c 6741 2007-07-27 19:26:53Z mike $"
  *
  *   Administration utility API definitions for the Common UNIX Printing
  *   System (CUPS).
  *
- *   MANY OF THE FUNCTIONS IN THIS HEADER ARE PRIVATE AND SUBJECT TO
- *   CHANGE AT ANY TIME.  USE AT YOUR OWN RISK.
- *
+ *   Copyright 2007 by Apple Inc.
  *   Copyright 2001-2007 by Easy Software Products.
  *
  *   These coded instructions, statements, and computer programs are the
- *   property of Easy Software Products and are protected by Federal
- *   copyright law.  Distribution and use rights are outlined in the file
- *   "LICENSE.txt" which should have been included with this file.  If this
- *   file is missing or damaged please contact Easy Software Products
- *   at:
- *
- *       Attn: CUPS Licensing Information
- *       Easy Software Products
- *       44141 Airport View Drive, Suite 204
- *       Hollywood, Maryland 20636 USA
- *
- *       Voice: (301) 373-9600
- *       EMail: cups-info@cups.org
- *         WWW: http://www.cups.org
+ *   property of Apple Inc. and are protected by Federal copyright
+ *   law.  Distribution and use rights are outlined in the file "LICENSE.txt"
+ *   which should have been included with this file.  If this file is
+ *   file is missing or damaged, see the license at "http://www.cups.org/".
  *
  *   This file is subject to the Apple OS-Developed Software exception.
  *
@@ -31,8 +19,10 @@
  *
  *   cupsAdminCreateWindowsPPD()   - Create the Windows PPD file for a printer.
  *   cupsAdminExportSamba()        - Export a printer to Samba.
- *   _cupsAdminGetServerSettings() - Get settings from the server.
- *   _cupsAdminSetServerSettings() - Set settings on the server.
+ *   cupsAdminGetServerSettings()  - Get settings from the server.
+ *   _cupsAdminGetServerSettings() - Get settings from the server (private).
+ *   cupsAdminSetServerSettings()  - Set settings on the server.
+ *   _cupsAdminSetServerSettings() - Set settings on the server (private).
  *   do_samba_command()            - Do a SAMBA command.
  *   get_cupsd_conf()              - Get the current cupsd.conf file.
  *   invalidate_cupsd_cache()      - Invalidate the cached cupsd.conf settings.
@@ -60,6 +50,12 @@
  * Local functions...
  */
 
+extern int		_cupsAdminGetServerSettings(http_t *http,
+				                    int *num_settings,
+			                            cups_option_t **settings);
+extern int		_cupsAdminSetServerSettings(http_t *http,
+			                            int num_settings,
+			                            cups_option_t *settings);
 static int		do_samba_command(const char *command,
 			                 const char *address,
 			                 const char *subcommand,
@@ -103,6 +99,7 @@ cupsAdminCreateWindowsPPD(
 			option[41],	/* Option */
 			choice[41];	/* Choice */
   int			jcloption,	/* In a JCL option? */
+			jclorder,	/* Next JCL order dependency */
 			linenum;	/* Current line number */
   time_t		curtime;	/* Current time */
   struct tm		*curdate;	/* Current date */
@@ -200,6 +197,7 @@ cupsAdminCreateWindowsPPD(
   */
 
   jcloption = 0;
+  jclorder  = 0;
   linenum   = 0;
   language  = cupsLangDefault();
 
@@ -238,10 +236,23 @@ cupsAdminCreateWindowsPPD(
       jcloption = 0;
       cupsFilePrintf(dstfp, "%s\n", line);
     }
+    else if (jcloption && !strncmp(line, "*OrderDependency:", 17))
+    {
+      for (ptr = line + 17; *ptr && isspace(*ptr & 255); ptr ++);
+
+      ptr = strchr(ptr, ' ');
+
+      if (ptr)
+      {
+	cupsFilePrintf(dstfp, "*OrderDependency: %d%s\n", jclorder, ptr);
+	jclorder ++;
+      }
+      else
+        cupsFilePrintf(dstfp, "%s\n", line);
+    }
     else if (jcloption &&
              strncmp(line, "*End", 4) &&
-             strncmp(line, "*Default", 8) &&
-             strncmp(line, "*OrderDependency", 16))
+             strncmp(line, "*Default", 8))
     {
       if ((ptr = strchr(line, ':')) == NULL)
       {
@@ -358,24 +369,24 @@ cupsAdminCreateWindowsPPD(
                                   IPP_TAG_ZERO)) != NULL &&
       (suppattr = ippFindAttribute(response, "job-hold-until-supported",
                                    IPP_TAG_ZERO)) != NULL)
-    write_option(dstfp, 10, "cupsJobHoldUntil", "Hold Until", "job-hold-until",
-                 suppattr, defattr, 0, 1);
+    write_option(dstfp, jclorder ++, "cupsJobHoldUntil", "Hold Until",
+                 "job-hold-until", suppattr, defattr, 0, 1);
 
   if ((defattr = ippFindAttribute(response, "job-priority-default",
                                   IPP_TAG_INTEGER)) != NULL &&
       (suppattr = ippFindAttribute(response, "job-priority-supported",
                                    IPP_TAG_RANGE)) != NULL)
-    write_option(dstfp, 11, "cupsJobPriority", "Priority", "job-priority",
-                 suppattr, defattr, 0, 1);
+    write_option(dstfp, jclorder ++, "cupsJobPriority", "Priority",
+                 "job-priority", suppattr, defattr, 0, 1);
 
   if ((defattr = ippFindAttribute(response, "job-sheets-default",
                                   IPP_TAG_ZERO)) != NULL &&
       (suppattr = ippFindAttribute(response, "job-sheets-supported",
                                    IPP_TAG_ZERO)) != NULL)
   {
-    write_option(dstfp, 20, "cupsJobSheetsStart", "Start Banner",
+    write_option(dstfp, jclorder ++, "cupsJobSheetsStart", "Start Banner",
                  "job-sheets", suppattr, defattr, 0, 2);
-    write_option(dstfp, 21, "cupsJobSheetsEnd", "End Banner",
+    write_option(dstfp, jclorder ++, "cupsJobSheetsEnd", "End Banner",
                  "job-sheets", suppattr, defattr, 1, 2);
   }
 
@@ -572,6 +583,10 @@ cupsAdminExportSamba(
     }
   }
 
+ /*
+  * See if we have the Win9x PS driver...
+  */
+
   snprintf(file, sizeof(file), "%s/drivers/ADOBEPS4.DRV", cg->cups_datadir);
   if (!access(file, 0))
   {
@@ -642,6 +657,136 @@ cupsAdminExportSamba(
     }
   }
 
+ /*
+  * See if we have the 64-bit Windows PS driver...
+  *
+  * Files:
+  *
+  *     x64/ps5ui.dll
+  *     x64/pscript.hlp
+  *     x64/pscript.ntf
+  *     x64/pscript5.dll
+  */
+
+  snprintf(file, sizeof(file), "%s/drivers/x64/pscript5.dll", cg->cups_datadir);
+  if (!access(file, 0))
+  {
+    have_drivers |= 4;
+
+   /*
+    * 64-bit Windows driver is installed; do the smbclient commands needed
+    * to copy the Win64 drivers over...
+    */
+
+    snprintf(address, sizeof(address), "//%s/print$", samba_server);
+
+    snprintf(subcmd, sizeof(subcmd),
+             "mkdir x64;"
+	     "put %s x64/%s.ppd;"
+	     "put %s/drivers/x64/ps5ui.dll x64/ps5ui.dll;"
+	     "put %s/drivers/x64/pscript.hlp x64/pscript.hlp;"
+	     "put %s/drivers/x64/pscript.ntf x64/pscript.ntf;"
+	     "put %s/drivers/x64/pscript5.dll x64/pscript5.dll",
+	     ppd, dest, cg->cups_datadir, cg->cups_datadir,
+	     cg->cups_datadir, cg->cups_datadir);
+
+    if ((status = do_samba_command("smbclient", address, subcmd,
+                                   authfile, logfile)) != 0)
+    {
+      snprintf(message, sizeof(message),
+               _cupsLangString(language,
+	                       _("Unable to copy 64-bit Windows printer "
+	                         "driver files (%d)!")), status);
+
+      _cupsSetError(IPP_INTERNAL_ERROR, message);
+
+      if (logfile)
+	_cupsLangPrintf(logfile, "%s\n", message);
+
+      unlink(authfile);
+
+      return (0);
+    }
+
+   /*
+    * See if we also have the CUPS driver files; if so, use them!
+    */
+
+    snprintf(file, sizeof(file), "%s/drivers/x64/cupsps6.dll", cg->cups_datadir);
+    if (!access(file, 0))
+    {
+     /*
+      * Copy the CUPS driver files over...
+      */
+
+      snprintf(subcmd, sizeof(subcmd),
+               "put %s/drivers/x64/cups6.ini x64/cups6.ini;"
+               "put %s/drivers/x64/cupsps6.dll x64/cupsps6.dll;"
+	       "put %s/drivers/x64/cupsui6.dll x64/cupsui6.dll",
+	       cg->cups_datadir, cg->cups_datadir, cg->cups_datadir);
+
+      if ((status = do_samba_command("smbclient", address, subcmd,
+                                     authfile, logfile)) != 0)
+      {
+	snprintf(message, sizeof(message),
+        	 _cupsLangString(language,
+	                         _("Unable to copy 64-bit CUPS printer driver "
+				   "files (%d)!")), status);
+
+	_cupsSetError(IPP_INTERNAL_ERROR, message);
+
+	if (logfile)
+	  _cupsLangPrintf(logfile, "%s\n", message);
+
+        unlink(authfile);
+
+	return (0);
+      }
+      
+     /*
+      * Do the rpcclient command needed for the CUPS drivers...
+      */
+
+      snprintf(subcmd, sizeof(subcmd),
+               "adddriver \"Windows x64\" \"%s:"
+	       "pscript5.dll:%s.ppd:ps5ui.dll:pscript.hlp:NULL:RAW:"
+	       "pscript5.dll,%s.ppd,ps5ui.dll,pscript.hlp,pscript.ntf,"
+	       "cups6.ini,cupsps6.dll,cupsui6.dll\"",
+	       dest, dest, dest);
+    }
+    else
+    {
+     /*
+      * Don't have the CUPS drivers, so just use the standard Windows
+      * drivers...
+      */
+
+      snprintf(subcmd, sizeof(subcmd),
+               "adddriver \"Windows x64\" \"%s:"
+	       "pscript5.dll:%s.ppd:ps5ui.dll:pscript.hlp:NULL:RAW:"
+	       "pscript5.dll,%s.ppd,ps5ui.dll,pscript.hlp,pscript.ntf\"",
+	       dest, dest, dest);
+    }
+
+    if ((status = do_samba_command("rpcclient", samba_server, subcmd,
+                                   authfile, logfile)) != 0)
+    {
+      snprintf(message, sizeof(message),
+               _cupsLangString(language,
+                	       _("Unable to install Windows 2000 printer "
+		        	 "driver files (%d)!")), status);
+
+      _cupsSetError(IPP_INTERNAL_ERROR, message);
+
+      if (logfile)
+	_cupsLangPrintf(logfile, "%s\n", message);
+
+      unlink(authfile);
+
+      return (0);
+    }
+  }
+
   if (logfile && !(have_drivers & 1))
   {
     if (!have_drivers)
@@ -696,6 +841,25 @@ cupsAdminExportSamba(
 }
 
 
+/*
+ * 'cupsAdminGetServerSettings()' - Get settings from the server.
+ *
+ * The returned settings should be freed with cupsFreeOptions() when
+ * you are done with them.
+ *
+ * @since CUPS 1.3@
+ */
+
+int					/* O - 1 on success, 0 on failure */
+cupsAdminGetServerSettings(
+    http_t        *http,		/* I - Connection to server */
+    int           *num_settings,	/* O - Number of settings */
+    cups_option_t **settings)		/* O - Settings */
+{
+  return (_cupsAdminGetServerSettings(http, num_settings, settings));
+}
+
+
 /*
  * '_cupsAdminGetServerSettings()' - Get settings from the server.
  *
@@ -771,6 +935,7 @@ _cupsAdminGetServerSettings(
 
     int		remote_access = 0,	/* Remote access allowed? */
 		remote_admin = 0,	/* Remote administration allowed? */
+		remote_any = 0,		/* Remote access from anywhere allowed? */
 		browsing = 1,		/* Browsing enabled? */
 		browse_allow = 1,	/* Browse address set? */
 		browse_address = 0,	/* Browse address set? */
@@ -790,7 +955,7 @@ _cupsAdminGetServerSettings(
 
     while (cupsFileGetConf(cupsd, line, sizeof(line), &value, &linenum))
     {
-      if (!value)
+      if (!value && strncmp(line, "</", 2))
         continue;
 
       if (!strcasecmp(line, "Port") || !strcasecmp(line, "Listen"))
@@ -903,6 +1068,9 @@ _cupsAdminGetServerSettings(
 	       )
       {
 	remote_admin = 1;
+
+	if (!strcasecmp(value, "all"))
+	  remote_any = 1;
       }
       else if (line[0] != '<' && !in_location && !in_policy)
         cg->cupsd_num_settings = cupsAddOption(line, value,
@@ -923,6 +1091,11 @@ _cupsAdminGetServerSettings(
 					   cg->cupsd_num_settings,
 					   &(cg->cupsd_settings));
 
+    cg->cupsd_num_settings = cupsAddOption(CUPS_SERVER_REMOTE_ANY,
+                                           remote_any ? "1" : "0",
+					   cg->cupsd_num_settings,
+					   &(cg->cupsd_settings));
+
     cg->cupsd_num_settings = cupsAddOption(CUPS_SERVER_REMOTE_PRINTERS,
                                            (browsing && browse_allow) ?
 					       "1" : "0",
@@ -960,6 +1133,22 @@ _cupsAdminGetServerSettings(
 }
 
 
+/*
+ * 'cupsAdminSetServerSettings()' - Set settings on the server.
+ *
+ * @since CUPS 1.3@
+ */
+
+int					/* O - 1 on success, 0 on failure */
+cupsAdminSetServerSettings(
+    http_t        *http,		/* I - Connection to server */
+    int           num_settings,		/* I - Number of settings */
+    cups_option_t *settings)		/* I - Settings */
+{
+  return (_cupsAdminSetServerSettings(http, num_settings, settings));
+}
+
+
 /*
  * '_cupsAdminSetServerSettings()' - Set settings on the server.
  *
@@ -995,6 +1184,7 @@ _cupsAdminSetServerSettings(
   int		remote_printers,	/* Show remote printers */
 		share_printers,		/* Share local printers */
 		remote_admin,		/* Remote administration allowed? */
+		remote_any,		/* Remote access from anywhere? */
 		user_cancel_any,	/* Cancel-job policy set? */
 		debug_logging;		/* LogLevel debug set? */
   int		wrote_port_listen,	/* Wrote the port/listen lines? */
@@ -1063,6 +1253,12 @@ _cupsAdminSetServerSettings(
   else
     old_remote_admin = 0;
 
+  if ((val = cupsGetOption(CUPS_SERVER_REMOTE_ANY, cupsd_num_settings,
+                           cupsd_settings)) != NULL)
+    remote_any = atoi(val);
+  else
+    remote_any = 0;
+
   if ((val = cupsGetOption(CUPS_SERVER_REMOTE_PRINTERS, cupsd_num_settings,
                            cupsd_settings)) != NULL)
     old_remote_printers = atoi(val);
@@ -1104,12 +1300,16 @@ _cupsAdminSetServerSettings(
   else
     debug_logging = -1;
 
+  if ((val = cupsGetOption(CUPS_SERVER_REMOTE_ANY, num_settings,
+                           settings)) != NULL)
+    remote_any = atoi(val);
+
   if ((val = cupsGetOption(CUPS_SERVER_REMOTE_ADMIN, num_settings,
                            settings)) != NULL)
   {
     remote_admin = atoi(val);
 
-    if (remote_admin == old_remote_admin)
+    if (remote_admin == old_remote_admin && remote_any < 0)
     {
      /*
       * No change to this setting...
@@ -1143,7 +1343,7 @@ _cupsAdminSetServerSettings(
   {
     share_printers = atoi(val);
 
-    if (share_printers == old_share_printers)
+    if (share_printers == old_share_printers && remote_any < 0)
     {
      /*
       * No change to this setting...
@@ -1288,7 +1488,7 @@ _cupsAdminSetServerSettings(
 	  cupsFilePuts(temp, "BrowseOrder allow,deny\n");
 
 	  if (new_remote_printers)
-	    cupsFilePuts(temp, "BrowseAllow @LOCAL\n");
+	    cupsFilePuts(temp, "BrowseAllow all\n");
 
 	  if (new_share_printers)
 	    cupsFilePuts(temp, "BrowseAddress @LOCAL\n");
@@ -1337,8 +1537,8 @@ _cupsAdminSetServerSettings(
 	                     "cancel a job...\n"
 	                     "  <Limit Cancel-Job>\n"
 	                     "    Order deny,allow\n"
-	                     "    Allow @SYSTEM\n"
-			     "    Allow @OWNER\n"
+			     "    Require user @OWNER "
+			     CUPS_DEFAULT_PRINTADMIN_AUTH "\n"
 			     "  </Limit>\n");
       }
 
@@ -1376,7 +1576,8 @@ _cupsAdminSetServerSettings(
         cupsFilePuts(temp, "  Order allow,deny\n");
 
 	if (remote_admin)
-	  cupsFilePuts(temp, "  Allow @LOCAL\n");
+	  cupsFilePrintf(temp, "  Allow %s\n",
+	                 remote_any > 0 ? "all" : "@LOCAL");
 	else
 	  cupsFilePuts(temp, "  Allow localhost\n");
       }
@@ -1394,7 +1595,8 @@ _cupsAdminSetServerSettings(
         cupsFilePuts(temp, "  Order allow,deny\n");
 
 	if (remote_admin)
-	  cupsFilePuts(temp, "  Allow @LOCAL\n");
+	  cupsFilePrintf(temp, "  Allow %s\n",
+	                 remote_any > 0 ? "all" : "@LOCAL");
 	else
 	  cupsFilePuts(temp, "  Allow localhost\n");
       }
@@ -1415,7 +1617,8 @@ _cupsAdminSetServerSettings(
         cupsFilePuts(temp, "  Order allow,deny\n");
 
 	if (remote_admin > 0 || share_printers > 0)
-	  cupsFilePuts(temp, "  Allow @LOCAL\n");
+	  cupsFilePrintf(temp, "  Allow %s\n",
+	                 remote_any > 0 ? "all" : "@LOCAL");
 	else
 	  cupsFilePuts(temp, "  Allow localhost\n");
       }
@@ -1487,7 +1690,8 @@ _cupsAdminSetServerSettings(
 	                   "a job...\n"
 	                   "  <Limit Cancel-Job>\n"
 	                   "    Order deny,allow\n"
-	                   "    Require user @OWNER @SYSTEM\n"
+	                   "    Require user @OWNER "
+			   CUPS_DEFAULT_PRINTADMIN_AUTH "\n"
 			   "  </Limit>\n");
 
       in_cancel_job = 0;
@@ -1573,7 +1777,7 @@ _cupsAdminSetServerSettings(
       cupsFilePuts(temp, "BrowseOrder allow,deny\n");
 
       if (remote_printers > 0)
-	cupsFilePuts(temp, "BrowseAllow @LOCAL\n");
+	cupsFilePuts(temp, "BrowseAllow all\n");
 
       if (share_printers > 0)
 	cupsFilePuts(temp, "BrowseAddress @LOCAL\n");
@@ -1635,7 +1839,7 @@ _cupsAdminSetServerSettings(
                        "  Order allow,deny\n");
 
     if (remote_admin > 0 || share_printers > 0)
-      cupsFilePuts(temp, "  Allow @LOCAL\n");
+      cupsFilePrintf(temp, "  Allow %s\n", remote_any > 0 ? "all" : "@LOCAL");
     else
       cupsFilePuts(temp, "  Allow localhost\n");
 
@@ -1653,7 +1857,7 @@ _cupsAdminSetServerSettings(
                        "  Order allow,deny\n");
 
     if (remote_admin)
-      cupsFilePuts(temp, "  Allow @LOCAL\n");
+      cupsFilePrintf(temp, "  Allow %s\n", remote_any > 0 ? "all" : "@LOCAL");
     else
       cupsFilePuts(temp, "  Allow localhost\n");
 
@@ -1674,7 +1878,7 @@ _cupsAdminSetServerSettings(
                        "  Order allow,deny\n");
 
     if (remote_admin)
-      cupsFilePuts(temp, "  Allow @LOCAL\n");
+      cupsFilePrintf(temp, "  Allow %s\n", remote_any > 0 ? "all" : "@LOCAL");
     else
       cupsFilePuts(temp, "  Allow localhost\n");
 
@@ -1685,7 +1889,7 @@ _cupsAdminSetServerSettings(
   {
     cupsFilePuts(temp, "<Policy default>\n"
                        "  # Job-related operations must be done by the owner "
-		       "or an adminstrator...\n"
+		       "or an administrator...\n"
                        "  <Limit Send-Document Send-URI Hold-Job Release-Job "
 		       "Restart-Job Purge-Jobs Set-Job-Attributes "
 		       "Create-Job-Subscription Renew-Subscription "
@@ -1696,7 +1900,7 @@ _cupsAdminSetServerSettings(
                        "    Order deny,allow\n"
                        "  </Limit>\n"
                        "  # All administration operations require an "
-		       "adminstrator to authenticate...\n"
+		       "administrator to authenticate...\n"
 		       "  <Limit Pause-Printer Resume-Printer "
                        "Set-Printer-Attributes Enable-Printer "
 		       "Disable-Printer Pause-Printer-After-Current-Job "
@@ -1716,8 +1920,9 @@ _cupsAdminSetServerSettings(
       cupsFilePuts(temp, "  # Only the owner or an administrator can cancel "
                          "a job...\n"
 	                 "  <Limit Cancel-Job>\n"
-	                 "    Require user @OWNER @SYSTEM\n"
 	                 "    Order deny,allow\n"
+	                 "    Require user @OWNER "
+			 CUPS_DEFAULT_PRINTADMIN_AUTH "\n"
 			 "  </Limit>\n");
 
     cupsFilePuts(temp, "  <Limit All>\n"
@@ -1778,6 +1983,10 @@ _cupsAdminSetServerSettings(
                                 	 old_remote_admin ? "1" : "0",
 					 cupsd_num_settings, &cupsd_settings);
 
+    cupsd_num_settings = cupsAddOption(CUPS_SERVER_REMOTE_ANY,
+                                       remote_any ? "1" : "0",
+				       cupsd_num_settings, &cupsd_settings);
+
     if (remote_printers >= 0)
       cupsd_num_settings = cupsAddOption(CUPS_SERVER_REMOTE_PRINTERS,
                                 	 remote_printers ? "1" : "0",
@@ -2137,5 +2346,5 @@ write_option(cups_file_t     *dstfp,	/* I - PPD file */
 
 
 /*
- * End of "$Id: adminutil.c 6625 2007-07-06 22:39:54Z mike $".
+ * End of "$Id: adminutil.c 6741 2007-07-27 19:26:53Z mike $".
  */
