@@ -1,25 +1,16 @@
 /*
- * "$Id: encode.c 6359 2007-03-19 14:46:09Z mike $"
+ * "$Id: encode.c 6649 2007-07-11 21:46:42Z mike $"
  *
  *   Option encoding routines for the Common UNIX Printing System (CUPS).
  *
+ *   Copyright 2007 by Apple Inc.
  *   Copyright 1997-2007 by Easy Software Products.
  *
  *   These coded instructions, statements, and computer programs are the
- *   property of Easy Software Products and are protected by Federal
- *   copyright law.  Distribution and use rights are outlined in the file
- *   "LICENSE.txt" which should have been included with this file.  If this
- *   file is missing or damaged please contact Easy Software Products
- *   at:
- *
- *       Attn: CUPS Licensing Information
- *       Easy Software Products
- *       44141 Airport View Drive, Suite 204
- *       Hollywood, Maryland 20636 USA
- *
- *       Voice: (301) 373-9600
- *       EMail: cups-info@cups.org
- *         WWW: http://www.cups.org
+ *   property of Apple Inc. and are protected by Federal copyright
+ *   law.  Distribution and use rights are outlined in the file "LICENSE.txt"
+ *   which should have been included with this file.  If this file is
+ *   file is missing or damaged, see the license at "http://www.cups.org/".
  *
  *   This file is subject to the Apple OS-Developed Software exception.
  *
@@ -52,6 +43,8 @@
 
 static const _ipp_option_t ipp_options[] =
 {
+  { "auth-info",		IPP_TAG_TEXT,		IPP_TAG_JOB },
+  { "auth-info-required",	IPP_TAG_KEYWORD,	IPP_TAG_PRINTER },
   { "blackplot",		IPP_TAG_BOOLEAN,	IPP_TAG_JOB },
   { "blackplot-default",	IPP_TAG_BOOLEAN,	IPP_TAG_PRINTER },
   { "brightness",		IPP_TAG_INTEGER,	IPP_TAG_JOB },
@@ -61,6 +54,7 @@ static const _ipp_option_t ipp_options[] =
   { "copies",			IPP_TAG_INTEGER,	IPP_TAG_JOB },
   { "copies-default",		IPP_TAG_INTEGER,	IPP_TAG_PRINTER },
   { "document-format",		IPP_TAG_MIMETYPE,	IPP_TAG_OPERATION },
+  { "document-format-default",	IPP_TAG_MIMETYPE,	IPP_TAG_PRINTER },
   { "finishings",		IPP_TAG_ENUM,		IPP_TAG_JOB },
   { "finishings-default",	IPP_TAG_ENUM,		IPP_TAG_PRINTER },
   { "fitplot",			IPP_TAG_BOOLEAN,	IPP_TAG_JOB },
@@ -82,7 +76,9 @@ static const _ipp_option_t ipp_options[] =
   { "natural-scaling-default",	IPP_TAG_INTEGER,	IPP_TAG_PRINTER },
   { "notify-charset",		IPP_TAG_CHARSET,	IPP_TAG_SUBSCRIPTION },
   { "notify-events",		IPP_TAG_KEYWORD,	IPP_TAG_SUBSCRIPTION },
+  { "notify-events-default",	IPP_TAG_KEYWORD,	IPP_TAG_PRINTER },
   { "notify-lease-duration",	IPP_TAG_INTEGER,	IPP_TAG_SUBSCRIPTION },
+  { "notify-lease-duration-default", IPP_TAG_INTEGER,	IPP_TAG_PRINTER },
   { "notify-natural-language",	IPP_TAG_LANGUAGE,	IPP_TAG_SUBSCRIPTION },
   { "notify-pull-method",	IPP_TAG_KEYWORD,	IPP_TAG_SUBSCRIPTION },
   { "notify-recipient-uri",	IPP_TAG_URI,		IPP_TAG_SUBSCRIPTION },
@@ -114,6 +110,7 @@ static const _ipp_option_t ipp_options[] =
   { "printer-info",		IPP_TAG_TEXT,		IPP_TAG_PRINTER },
   { "printer-is-accepting-jobs",IPP_TAG_BOOLEAN,	IPP_TAG_PRINTER },
   { "printer-is-shared",	IPP_TAG_BOOLEAN,	IPP_TAG_PRINTER },
+  { "printer-location",		IPP_TAG_TEXT,		IPP_TAG_PRINTER },
   { "printer-make-and-model",	IPP_TAG_TEXT,		IPP_TAG_PRINTER },
   { "printer-more-info",	IPP_TAG_URI,		IPP_TAG_PRINTER },
   { "printer-op-policy",	IPP_TAG_NAME,		IPP_TAG_PRINTER },
@@ -567,5 +564,5 @@ compare_ipp_options(_ipp_option_t *a,	/* I - First option */
 
 
 /*
- * End of "$Id: encode.c 6359 2007-03-19 14:46:09Z mike $".
+ * End of "$Id: encode.c 6649 2007-07-11 21:46:42Z mike $".
  */
