@@ -1,25 +1,16 @@
 /*
- * "$Id: http-support.c 6188 2007-01-10 16:23:06Z mike $"
+ * "$Id: http-support.c 7030 2007-10-19 01:23:05Z mike $"
  *
  *   HTTP support routines for the Common UNIX Printing System (CUPS) scheduler.
  *
+ *   Copyright 2007 by Apple Inc.
  *   Copyright 1997-2007 by Easy Software Products, all rights reserved.
  *
  *   These coded instructions, statements, and computer programs are the
- *   property of Easy Software Products and are protected by Federal
- *   copyright law.  Distribution and use rights are outlined in the file
- *   "LICENSE.txt" which should have been included with this file.  If this
- *   file is missing or damaged please contact Easy Software Products
- *   at:
- *
- *       Attn: CUPS Licensing Information
- *       Easy Software Products
- *       44141 Airport View Drive, Suite 204
- *       Hollywood, Maryland 20636 USA
- *
- *       Voice: (301) 373-9600
- *       EMail: cups-info@cups.org
- *         WWW: http://www.cups.org
+ *   property of Apple Inc. and are protected by Federal copyright
+ *   law.  Distribution and use rights are outlined in the file "LICENSE.txt"
+ *   which should have been included with this file.  If this file is
+ *   file is missing or damaged, see the license at "http://www.cups.org/".
  *
  *   This file is subject to the Apple OS-Developed Software exception.
  *
@@ -149,7 +140,7 @@ httpAssembleURI(
   if (!ptr)
     goto assemble_overflow;
 
-  if (!strcmp(scheme, "mailto:"))
+  if (!strcmp(scheme, "mailto"))
   {
    /*
     * mailto: only has :, no //...
@@ -1025,7 +1016,7 @@ httpSeparateURI(
 			 "0123456789"
 	        	 "-._~"
 			 "%"
-			 "!$&'()*+,;=", *ptr))
+			 "!$&'()*+,;=\\", *ptr))
 	{
 	  *host = '\0';
 	  return (HTTP_URI_BAD_HOSTNAME);
@@ -1327,5 +1318,5 @@ http_copy_encode(char       *dst,	/* O - Destination buffer */
 
 
 /*
- * End of "$Id: http-support.c 6188 2007-01-10 16:23:06Z mike $".
+ * End of "$Id: http-support.c 7030 2007-10-19 01:23:05Z mike $".
  */
