@@ -1,25 +1,16 @@
 /*
- * "$Id: main.c 6516 2007-05-07 19:10:54Z mike $"
+ * "$Id: main.c 7045 2007-10-31 05:42:13Z mike $"
  *
  *   Scheduler main loop for the Common UNIX Printing System (CUPS).
  *
+ *   Copyright 2007 by Apple Inc.
  *   Copyright 1997-2007 by Easy Software Products, all rights reserved.
  *
  *   These coded instructions, statements, and computer programs are the
- *   property of Easy Software Products and are protected by Federal
- *   copyright law.  Distribution and use rights are outlined in the file
+ *   property of Apple Inc. and are protected by Federal copyright
+ *   law.  Distribution and use rights are outlined in the file "LICENSE.txt"
  *   "LICENSE" which should have been included with this file.  If this
- *   file is missing or damaged please contact Easy Software Products
- *   at:
- *
- *       Attn: CUPS Licensing Information
- *       Easy Software Products
- *       44141 Airport View Drive, Suite 204
- *       Hollywood, Maryland 20636 USA
- *
- *       Voice: (301) 373-9600
- *       EMail: cups-info@cups.org
- *         WWW: http://www.cups.org
+ *   file is missing or damaged, see the license at "http://www.cups.org/".
  *
  * Contents:
  *
@@ -53,6 +44,7 @@
 #include <sys/resource.h>
 #include <syslog.h>
 #include <grp.h>
+#include <cups/dir.h>
 
 #ifdef HAVE_LAUNCH_H
 #  include <launch.h>
@@ -77,6 +69,10 @@
 #  include <notify.h>
 #endif /* HAVE_NOTIFY_H */
 
+#if defined(__APPLE__) && defined(HAVE_DLFCN_H)
+#  include <dlfcn.h>
+#endif /* __APPLE__ && HAVE_DLFCN_H */
+
 
 /*
  * Local functions...
@@ -110,6 +106,12 @@ static int		dead_children = 0;
 static int		stop_scheduler = 0;
 					/* Should the scheduler stop? */
 
+#if defined(__APPLE__) && defined(HAVE_DLFCN_H)
+static const char *PSQLibPath = "/usr/lib/libPrintServiceQuota.dylib";
+static const char *PSQLibFuncName = "PSQUpdateQuota";
+static void *PSQLibRef;			/* libPrintServiceQuota.dylib */
+#endif /* HAVE_DLFCN_H */
+
 
 /*
  * 'main()' - Main entry for the CUPS scheduler.
@@ -123,8 +125,6 @@ main(int  argc,				/* I - Number of command-line args */
   char			*opt;		/* Option character */
   int			fg;		/* Run in the foreground */
   int			fds;		/* Number of ready descriptors */
-  fd_set		*input,		/* Input set for select() */
-			*output;	/* Output set for select() */
   cupsd_client_t	*con;		/* Current client */
   cupsd_job_t		*job;		/* Current job */
   cupsd_listener_t	*lis;		/* Current listener */
@@ -133,14 +133,11 @@ main(int  argc,				/* I - Number of command-line args */
 			browse_time,	/* Next browse send time */
 			senddoc_time,	/* Send-Document time */
 			expire_time,	/* Subscription expire time */
-#ifndef __APPLE__
-			netif_time,	/* Network interface poll time */
-#endif /* !__APPLE__ */
 			mallinfo_time;	/* Malloc information time */
   size_t		string_count,	/* String count */
 			alloc_bytes,	/* Allocated string bytes */
 			total_bytes;	/* Total string bytes */
-  struct timeval	timeout;	/* select() timeout */
+  long			timeout;	/* Timeout for cupsdDoSelect() */
   struct rlimit		limit;		/* Runtime limit */
 #if defined(HAVE_SIGACTION) && !defined(HAVE_SIGSET)
   struct sigaction	action;		/* Actions for POSIX signals */
@@ -155,12 +152,32 @@ main(int  argc,				/* I - Number of command-line args */
 #endif	/* HAVE_LAUNCHD */
 
 
+#ifdef HAVE_GETEUID
+ /*
+  * Check for setuid invocation, which we do not support!
+  */
+
+  if (getuid() != geteuid())
+  {
+    fputs("cupsd: Cannot run as a setuid program!\n", stderr);
+    return (1);
+  }
+#endif /* HAVE_GETEUID */
+
  /*
   * Check for command-line arguments...
   */
 
   fg = 0;
 
+#ifdef HAVE_LAUNCHD
+  if (getenv("CUPSD_LAUNCHD"))
+  {
+    Launchd = 1;
+    fg      = 1;
+  }
+#endif /* HAVE_LAUNCHD */
+
   for (i = 1; i < argc; i ++)
     if (argv[i][0] == '-')
       for (opt = argv[i] + 1; *opt != '\0'; opt ++)
@@ -369,33 +386,23 @@ main(int  argc,				/* I - Number of command-line args */
 
   getrlimit(RLIMIT_NOFILE, &limit);
 
+#if !defined(HAVE_POLL) && !defined(HAVE_EPOLL) && !defined(HAVE_KQUEUE)
   if (limit.rlim_max > FD_SETSIZE)
     MaxFDs = FD_SETSIZE;
   else
+#endif /* !HAVE_POLL && !HAVE_EPOLL && !HAVE_KQUEUE */
+#ifdef RLIM_INFINITY
+  if (limit.rlim_max == RLIM_INFINITY)
+    MaxFDs = 16384;
+  else
+#endif /* RLIM_INFINITY */
     MaxFDs = limit.rlim_max;
 
   limit.rlim_cur = MaxFDs;
 
   setrlimit(RLIMIT_NOFILE, &limit);
 
- /*
-  * Allocate memory for the input and output sets...
-  */
-
-  SetSize = (MaxFDs + 31) / 8 + 4;
-  if (SetSize < sizeof(fd_set))
-    SetSize = sizeof(fd_set);
-
-  InputSet  = (fd_set *)calloc(1, SetSize);
-  OutputSet = (fd_set *)calloc(1, SetSize);
-  input     = (fd_set *)calloc(1, SetSize);
-  output    = (fd_set *)calloc(1, SetSize);
-
-  if (InputSet == NULL || OutputSet == NULL || input == NULL || output == NULL)
-  {
-    syslog(LOG_LPR, "Unable to allocate memory for select() sets - exiting!");
-    return (1);
-  }
+  cupsdStartSelect();
 
  /*
   * Read configuration...
@@ -408,6 +415,43 @@ main(int  argc,				/* I - Number of command-line args */
     return (1);
   }
 
+  if (!strncmp(TempDir, RequestRoot, strlen(RequestRoot)))
+  {
+   /*
+    * Clean out the temporary directory...
+    */
+
+    cups_dir_t		*dir;		/* Temporary directory */
+    cups_dentry_t	*dent;		/* Directory entry */
+    char		tempfile[1024];	/* Temporary filename */
+
+
+    if ((dir = cupsDirOpen(TempDir)) != NULL)
+    {
+      cupsdLogMessage(CUPSD_LOG_INFO,
+                      "Cleaning out old temporary files in \"%s\"...", TempDir);
+
+      while ((dent = cupsDirRead(dir)) != NULL)
+      {
+        snprintf(tempfile, sizeof(tempfile), "%s/%s", TempDir, dent->filename);
+
+	if (cupsdRemoveFile(tempfile))
+	  cupsdLogMessage(CUPSD_LOG_ERROR,
+	                  "Unable to remove temporary file \"%s\" - %s",
+	                  tempfile, strerror(errno));
+        else
+	  cupsdLogMessage(CUPSD_LOG_DEBUG, "Removed temporary file \"%s\"...",
+	                  tempfile);
+      }
+
+      cupsDirClose(dir);
+    }
+    else
+      cupsdLogMessage(CUPSD_LOG_ERROR,
+                      "Unable to open temporary directory \"%s\" - %s",
+                      TempDir, strerror(errno));
+  }
+
 #if HAVE_LAUNCHD
   if (Launchd)
   {
@@ -419,6 +463,39 @@ main(int  argc,				/* I - Number of command-line args */
   }
 #endif /* HAVE_LAUNCHD */
 
+#if defined(__APPLE__) && defined(HAVE_DLFCN_H)
+ /*
+  * Load Print Service quota enforcement library (X Server only)
+  */
+
+  PSQLibRef = dlopen(PSQLibPath, RTLD_LAZY);
+
+  if (PSQLibRef)
+    PSQUpdateQuotaProc = dlsym(PSQLibRef, PSQLibFuncName);
+#endif /* __APPLE__ && HAVE_DLFCN_H */
+
+#ifdef HAVE_GSSAPI
+#  ifdef __APPLE__
+ /*
+  * If the weak-linked GSSAPI/Kerberos library is not present, don't try
+  * to use it...
+  */
+
+  if (krb5_init_context != NULL)
+#  endif /* __APPLE__ */
+
+ /*
+  * Setup a Kerberos context for the scheduler to use...
+  */
+
+  if (krb5_init_context(&KerberosContext))
+  {
+    KerberosContext = NULL;
+
+    cupsdLogMessage(CUPSD_LOG_ERROR, "Unable to initialize Kerberos context");
+  }
+#endif /* HAVE_GSSAPI */
+
  /*
   * Startup the server...
   */
@@ -534,9 +611,6 @@ main(int  argc,				/* I - Number of command-line args */
   senddoc_time  = time(NULL);
   expire_time   = time(NULL);
   fds           = 1;
-#ifndef __APPLE__
-  netif_time    = 0;
-#endif /* !__APPLE__ */
 
   while (!stop_scheduler)
   {
@@ -632,18 +706,15 @@ main(int  argc,				/* I - Number of command-line args */
     }
 
    /*
-    * Check for available input or ready output.  If select() returns
-    * 0 or -1, something bad happened and we should exit immediately.
+    * Check for available input or ready output.  If cupsdDoSelect()
+    * returns 0 or -1, something bad happened and we should exit
+    * immediately.
     *
     * Note that we at least have one listening socket open at all
     * times.
     */
 
-    memcpy(input, InputSet, SetSize);
-    memcpy(output, OutputSet, SetSize);
-
-    timeout.tv_sec  = select_timeout(fds);
-    timeout.tv_usec = 0;
+    timeout = select_timeout(fds);
 
 #if HAVE_LAUNCHD
    /*
@@ -652,73 +723,41 @@ main(int  argc,				/* I - Number of command-line args */
     * inactivity...
     */
 
-    if (timeout.tv_sec == 86400 && Launchd && LaunchdTimeout && !NumPolled &&
+    if (timeout == 86400 && Launchd && LaunchdTimeout && !NumPolled &&
+        !cupsArrayCount(ActiveJobs) &&
 	(!Browsing || 
 	 (!BrowseRemoteProtocols && 
 	  (!NumBrowsers || !BrowseLocalProtocols ||
 	   cupsArrayCount(Printers) == 0))))
     {
-      timeout.tv_sec    = LaunchdTimeout;
+      timeout		= LaunchdTimeout;
       launchd_idle_exit = 1;
     }
     else
       launchd_idle_exit = 0;
 #endif	/* HAVE_LAUNCHD */
 
-    if (timeout.tv_sec < 86400)		/* Only use timeout for < 1 day */
-      fds = select(MaxFDs, input, output, NULL, &timeout);
-    else
-      fds = select(MaxFDs, input, output, NULL, NULL);
-
-    if (fds < 0)
+    if ((fds = cupsdDoSelect(timeout)) < 0)
     {
-      char	s[16384],		/* String buffer */
-		*sptr;			/* Pointer into buffer */
-      int	slen;			/* Length of string buffer */
-
-
      /*
       * Got an error from select!
       */
 
-      if (errno == EINTR)	/* Just interrupted by a signal */
+#ifdef HAVE_DNSSD
+      cupsd_printer_t	*p;		/* Current printer */
+#endif /* HAVE_DNSSD */
+
+
+      if (errno == EINTR)		/* Just interrupted by a signal */
         continue;
 
      /*
       * Log all sorts of debug info to help track down the problem.
       */
 
-      cupsdLogMessage(CUPSD_LOG_EMERG, "select() failed - %s!",
+      cupsdLogMessage(CUPSD_LOG_EMERG, "cupsdDoSelect() failed - %s!",
                       strerror(errno));
 
-      strcpy(s, "InputSet =");
-      slen = 10;
-      sptr = s + 10;
-
-      for (i = 0; i < MaxFDs; i ++)
-        if (FD_ISSET(i, InputSet))
-	{
-          snprintf(sptr, sizeof(s) - slen, " %d", i);
-	  slen += strlen(sptr);
-	  sptr += strlen(sptr);
-	}
-
-      cupsdLogMessage(CUPSD_LOG_EMERG, "%s", s);
-
-      strcpy(s, "OutputSet =");
-      slen = 11;
-      sptr = s + 11;
-
-      for (i = 0; i < MaxFDs; i ++)
-        if (FD_ISSET(i, OutputSet))
-	{
-          snprintf(sptr, sizeof(s) - slen, " %d", i);
-	  slen += strlen(sptr);
-	  sptr += strlen(sptr);
-	}
-
-      cupsdLogMessage(CUPSD_LOG_EMERG, "%s", s);
-
       for (i = 0, con = (cupsd_client_t *)cupsArrayFirst(Clients);
 	   con;
 	   i ++, con = (cupsd_client_t *)cupsArrayNext(Clients))
@@ -748,6 +787,15 @@ main(int  argc,				/* I - Number of command-line args */
 			job->status_buffer ? job->status_buffer->fd : -1,
 			job->print_pipes[0], job->print_pipes[1],
 			job->back_pipes[0], job->back_pipes[1]);
+
+#ifdef HAVE_DNSSD
+      for (p = (cupsd_printer_t *)cupsArrayFirst(Printers);
+	   p;
+	   p = (cupsd_printer_t *)cupsArrayNext(Printers))
+        cupsdLogMessage(CUPSD_LOG_EMERG, "printer[%s] %d", p->name,
+	                p->dnssd_ipp_fd);
+#endif /* HAVE_DNSSD */
+
       break;
     }
 
@@ -770,65 +818,12 @@ main(int  argc,				/* I - Number of command-line args */
 #endif /* HAVE_LAUNCHD */
 
    /*
-    * Check for status info from job filters...
+    * Resume listening for new connections as needed...
     */
 
-    for (job = (cupsd_job_t *)cupsArrayFirst(ActiveJobs);
-	 job;
-	 job = (cupsd_job_t *)cupsArrayNext(ActiveJobs))
-      if (job->status_buffer && FD_ISSET(job->status_buffer->fd, input))
-      {
-       /*
-        * Clear the input bit to avoid updating the next job
-	* using the same status pipe file descriptor...
-	*/
-
-        FD_CLR(job->status_buffer->fd, input);
-
-       /*
-        * Read any status messages from the filters...
-	*/
-
-        cupsdUpdateJob(job);
-      }
-
-   /*
-    * Update CGI messages as needed...
-    */
-
-    if (CGIPipes[0] >= 0 && FD_ISSET(CGIPipes[0], input))
-      cupsdUpdateCGI();
-
-   /*
-    * Handle system management events as needed...
-    */
-
-#ifdef __APPLE__
-   /*
-    * Mac OS X provides the SystemConfiguration framework for system
-    * configuration change events...
-    */
-
-    if (SysEventPipes[0] >= 0 && FD_ISSET(SysEventPipes[0], input))
-      cupsdUpdateSystemMonitor();
-#else
-   /*
-    * All other operating systems need to poll for changes...
-    */
-
-    if ((current_time - netif_time) >= 60)
-    {
-      NetIFUpdate = 1;
-      netif_time  = current_time;
-    }
-#endif	/* __APPLE__ */
-
-   /*
-    * Update notifier messages as needed...
-    */
-
-    if (NotifierPipes[0] >= 0 && FD_ISSET(NotifierPipes[0], input))
-      cupsdUpdateNotifierStatus();
+    if (ListeningPaused && ListeningPaused <= current_time &&
+        cupsArrayCount(Clients) < MaxClients)
+      cupsdResumeListening();
 
    /*
     * Expire subscriptions and unload completed jobs as needed...
@@ -848,14 +843,8 @@ main(int  argc,				/* I - Number of command-line args */
     * Update the browse list as needed...
     */
 
-    if (Browsing && BrowseRemoteProtocols)
+    if (Browsing)
     {
-      if (BrowseSocket >= 0 && FD_ISSET(BrowseSocket, input))
-        cupsdUpdateCUPSBrowse();
-
-      if (PollPipe >= 0 && FD_ISSET(PollPipe, input))
-        cupsdUpdatePolling();
-
 #ifdef HAVE_LIBSLP
       if ((BrowseRemoteProtocols & BROWSE_SLP) &&
           BrowseSLPRefresh <= current_time)
@@ -876,17 +865,20 @@ main(int  argc,				/* I - Number of command-line args */
     }
 
    /*
-    * Check for new connections on the "listen" sockets...
+    * Update the root certificate once every 5 minutes if we have client
+    * connections...
     */
 
-    for (lis = (cupsd_listener_t *)cupsArrayFirst(Listeners);
-         lis;
-	 lis = (cupsd_listener_t *)cupsArrayNext(Listeners))
-      if (lis->fd >= 0 && FD_ISSET(lis->fd, input))
-      {
-        FD_CLR(lis->fd, input);
-        cupsdAcceptClient(lis);
-      }
+    if ((current_time - RootCertTime) >= RootCertDuration && RootCertDuration &&
+        !RunUser && cupsArrayCount(Clients))
+    {
+     /*
+      * Update the root certificate...
+      */
+
+      cupsdDeleteCert(0);
+      cupsdAddCert(0, "root");
+    }
 
    /*
     * Check for new data on the client sockets...
@@ -897,61 +889,13 @@ main(int  argc,				/* I - Number of command-line args */
 	 con = (cupsd_client_t *)cupsArrayNext(Clients))
     {
      /*
-      * Process the input buffer...
+      * Process pending data in the input buffer...
       */
 
-      if (FD_ISSET(con->http.fd, input) || con->http.used)
+      if (con->http.used)
       {
-        int fd = con->file;
-
-
-        FD_CLR(con->http.fd, input);
-
-        if (!cupsdReadClient(con))
-	{
-	  if (fd >= 0)
-	    FD_CLR(fd, input);
-
-	  continue;
-	}
-      }
-
-     /*
-      * Write data as needed...
-      */
-
-      if (con->pipe_pid && FD_ISSET(con->file, input))
-      {
-       /*
-        * Keep track of pending input from the file/pipe separately
-	* so that we don't needlessly spin on select() when the web
-	* client is not ready to receive data...
-	*/
-
-	FD_CLR(con->file, input);
-        con->file_ready = 1;
-
-#ifdef DEBUG
-        cupsdLogMessage(CUPSD_LOG_DEBUG2, "main: Data ready file %d!",
-	                con->file);
-#endif /* DEBUG */
-
-	if (!FD_ISSET(con->http.fd, output))
-	{
-	  cupsdLogMessage(CUPSD_LOG_DEBUG2,
-	                  "main: Removing fd %d from InputSet...", con->file);
-	  FD_CLR(con->file, input);
-	  FD_CLR(con->file, InputSet);
-	}
-      }
-
-      if (FD_ISSET(con->http.fd, output))
-      {
-        FD_CLR(con->http.fd, output);
-
-	if (!con->pipe_pid || con->file_ready)
-          if (!cupsdWriteClient(con))
-	    continue;
+        cupsdReadClient(con);
+	continue;
       }
 
      /*
@@ -1007,22 +951,6 @@ main(int  argc,				/* I - Number of command-line args */
       mallinfo_time = current_time;
     }
 
-   /*
-    * Update the root certificate once every 5 minutes if we have client
-    * connections...
-    */
-
-    if ((current_time - RootCertTime) >= RootCertDuration && RootCertDuration &&
-        !RunUser && cupsArrayCount(Clients))
-    {
-     /*
-      * Update the root certificate...
-      */
-
-      cupsdDeleteCert(0);
-      cupsdAddCert(0, "root");
-    }
-
    /*
     * Handle OS-specific event notification for any events that have
     * accumulated.  Don't send these more than once a second...
@@ -1074,17 +1002,11 @@ main(int  argc,				/* I - Number of command-line args */
                     "Scheduler shutting down due to program error.");
 
  /*
-  * Close all network clients and stop all jobs...
+  * Close all network clients...
   */
 
   cupsdStopServer();
 
-  cupsdFreeAllJobs();
-
-#ifdef __APPLE__
-  cupsdStopSystemMonitor();
-#endif /* __APPLE__ */
-
 #ifdef HAVE_LAUNCHD
  /*
   * Update the launchd KeepAlive file as needed...
@@ -1094,6 +1016,52 @@ main(int  argc,				/* I - Number of command-line args */
     launchd_checkout();
 #endif /* HAVE_LAUNCHD */
 
+ /*
+  * Stop all jobs...
+  */
+
+  cupsdFreeAllJobs();
+
+#ifdef __APPLE__
+ /*
+  * Stop monitoring system event monitoring...
+  */
+
+  cupsdStopSystemMonitor();
+#endif /* __APPLE__ */
+
+#ifdef HAVE_GSSAPI
+ /*
+  * Free the scheduler's Kerberos context...
+  */
+
+#  ifdef __APPLE__
+ /*
+  * If the weak-linked GSSAPI/Kerberos library is not present, don't try
+  * to use it...
+  */
+
+  if (krb5_init_context != NULL)
+#  endif /* __APPLE__ */
+  if (KerberosContext)
+    krb5_free_context(KerberosContext);
+#endif /* HAVE_GSSAPI */
+
+#ifdef __APPLE__
+#ifdef HAVE_DLFCN_H
+ /* 
+  * Unload Print Service quota enforcement library (X Server only) 
+  */
+
+  PSQUpdateQuotaProc = NULL;
+  if (PSQLibRef)
+  {
+    dlclose(PSQLibRef);
+    PSQLibRef = NULL;
+  }
+#endif /* HAVE_DLFCN_H */
+#endif	/* __APPLE__ */
+
 #ifdef __sgi
  /*
   * Remove the fake IRIX lpsched lock file, but only if the existing
@@ -1106,14 +1074,7 @@ main(int  argc,				/* I - Number of command-line args */
       unlink("/var/spool/lp/SCHEDLOCK");
 #endif /* __sgi */
 
- /*
-  * Free memory used by FD sets and return...
-  */
-
-  free(InputSet);
-  free(OutputSet);
-  free(input);
-  free(output);
+  cupsdStopSelect();
 
   return (!stop_scheduler);
 }
@@ -1755,12 +1716,12 @@ select_timeout(int fds)			/* I - Number of descriptors returned */
       return (0);
 
  /*
-  * If select has been active in the last second (fds != 0) or we have
+  * If select has been active in the last second (fds > 0) or we have
   * many resources in use then don't bother trying to optimize the
   * timeout, just make it 1 second.
   */
 
-  if (fds || cupsArrayCount(Clients) > 50)
+  if (fds > 0 || cupsArrayCount(Clients) > 50)
     return (1);
 
  /*
@@ -1771,6 +1732,21 @@ select_timeout(int fds)			/* I - Number of descriptors returned */
   timeout = now + 86400;		/* 86400 == 1 day */
   why     = "do nothing";
 
+ /*
+  * Check whether we are accepting new connections...
+  */
+
+  if (ListeningPaused > 0 && cupsArrayCount(Clients) < MaxClients &&
+      ListeningPaused < timeout)
+  {
+    if (ListeningPaused <= now)
+      timeout = now;
+    else
+      timeout = ListeningPaused;
+
+    why = "resume listening";
+  }
+
  /*
   * Check the activity and close old clients...
   */
@@ -1806,7 +1782,7 @@ select_timeout(int fds)			/* I - Number of descriptors returned */
     }
 #endif /* HAVE_LDAP */
 
-    if (BrowseLocalProtocols & BROWSE_CUPS)
+    if ((BrowseLocalProtocols & BROWSE_CUPS) && NumBrowsers)
     {
       for (p = (cupsd_printer_t *)cupsArrayFirst(Printers);
            p;
@@ -1820,7 +1796,7 @@ select_timeout(int fds)			/* I - Number of descriptors returned */
 	    why     = "browse timeout a printer";
 	  }
 	}
-	else if (!(p->type & CUPS_PRINTER_IMPLICIT))
+	else if (p->shared && !(p->type & CUPS_PRINTER_IMPLICIT))
 	{
 	  if (BrowseInterval && (p->browse_time + BrowseInterval) < timeout)
 	  {
@@ -1894,8 +1870,8 @@ select_timeout(int fds)			/* I - Number of descriptors returned */
   * Log and return the timeout value...
   */
 
-  cupsdLogMessage(CUPSD_LOG_DEBUG2, "select_timeout: %ld seconds to %s",
-                  timeout, why);
+  cupsdLogMessage(CUPSD_LOG_DEBUG2, "select_timeout(%d): %ld seconds to %s",
+                  fds, timeout, why);
 
   return (timeout);
 }
@@ -1921,5 +1897,5 @@ usage(int status)			/* O - Exit status */
 
 
 /*
- * End of "$Id: main.c 6516 2007-05-07 19:10:54Z mike $".
+ * End of "$Id: main.c 7045 2007-10-31 05:42:13Z mike $".
  */
